<?php
namespace WPFunnels\Admin\FunnelType;
use WPFunnels\lms\helper\Wpfnl_lms_learndash_functions;
use WPFunnels\Modules\Frontend\LearnDash\Wpfnl_Frontend_LearnDash;
use WPFunnels\Wpfnl_functions;

class Wpfunnels_Lms_Checkout {

    /**
     * Constructor for the class.
     *
     * This constructor sets up various filters to modify the behavior and appearance
     * of the WordPress admin and LearnDash functionality.
     * @since 1.0.11
     */
    public function __construct() {
        add_filter( 'wpfunnels/checkout_columns', [ $this,'change_checkout_columns_in_admin' ], 10, 2 );
        add_filter( 'wpfunnels/ob_positions', [ $this,'change_ob_positions_in_admin' ], 10, 2 );
        add_filter( 'learndash_login_model_register_url', [ $this,'modify_learndash_register_url' ] );
    }

    /**
     * Modify the LearnDash registration URL by adding the 'wpfnl_step_id' query parameter.
     *
     * This function takes the original LearnDash registration URL and adds the 'wpfnl_step_id'
     * query parameter if a valid step ID is available.
     *
     * @param string $register_url The original LearnDash registration URL.
     *
     * @return string The modified registration URL with the 'wpfnl_step_id' query parameter added, if applicable.
     * @since 1.0.11
     */
    public function modify_learndash_register_url( $register_url ) {
        // Get the WP Funnel step ID.
        $step_id = Wpfnl_Frontend_LearnDash::get_wpfnl_step_id();

        // Check if a valid step ID is available.
        if( !empty( $step_id ) ) {
            // Add the 'wpfnl_step_id' query parameter to the registration URL.
            return add_query_arg( [ 'wpfnl_step_id' => $step_id ], $register_url );
        }

        // Return the original registration URL if no step ID is available.
        return $register_url;
    }

    /**
     * Retrieve and format a list of courses based on a search term.
     *
     * This function searches for courses using a given search term and retrieves relevant course information.
     * It filters out courses with 'open' or 'closed' pricing types and formats the results into an associative array.
     *
     * @note This function is called dynamically in wpfunnels/admin/modules/product/class-wpfnl-product.php in fetch_products() function.
     *
     * @param string $term The search term used to filter courses.
     *
     * @return array An associative array containing course details that match the search term.
     *               The array key is the course ID, and the value is an array with the following details:
     *               - 'name' (string): The name of the course.
     *               - 'price' (string): The course price.
     *               - 'sale_price' (string): The course sale price (same as the regular price in this context).
     * @since 1.0.11
     */
    public function retrieve_item( $term = '' ) {
        $formatted_courses = [];

        // Check if a search term is provided and if LearnDash is active.
        if( $term && Wpfnl_lms_learndash_functions::is_learndash_active() ) {
            // Search for courses based on the provided term.
            $courses = Wpfnl_lms_learndash_functions::search_course( $term );

            // Process and format the retrieved courses.
            if( !empty( $courses ) ) {
                foreach( $courses as $key => $course ) {
                    $course_meta = learndash_get_course_meta_setting( $course->ID );

                    // Skip courses with 'open' or 'closed' pricing types.
                    if( $course_meta[ 'sfwd-courses_course_price_type' ] === 'open' || $course_meta[ 'sfwd-courses_course_price_type' ] === 'closed' ) {
                        continue;
                    }

                    // Get the course price.
                    $price = function_exists( 'learndash_get_course_price' ) ? learndash_get_course_price( $course ) : '';
                    $price = isset( $price[ 'price' ] ) ? $price[ 'price' ] : $price;

                    // Format course details and add to the result array.
                    $formatted_courses[ $course->ID ] = [
                        'name'       => $course->post_title,
                        'price'      => $price,
                        'sale_price' => $price, // In this context, sale_price is the same as the regular price.
                    ];
                }
            }
        }

        return $formatted_courses;
    }

    /**
     * Save items to a list of saved products.
     *
     * This function checks if a product with a given ID already exists in the list of saved products.
     * If the product is not already saved, it retrieves details about the product/course and adds it to the list.
     *
     * @note This function is called dynamically in wpfunnels/includes/core/rest-api/Controllers/class-checkout-controller.php in add_product() function.
     *
     * @param array   $payload        The payload containing information about the product.
     * @param int     $product_id     The ID of the product/course to be saved.
     * @param array   $saved_products The list of previously saved products.
     *
     * @return array An associative array containing information about the saved product or an error message.
     *               - 'success' (bool): Indicates whether the product was successfully saved.
     *               - 'message' (string): An informative message about the result.
     *               - 'products' (array): An array containing details of the saved product, if successful.
     *                                      Otherwise, it's an empty array.
     * @since 1.0.11
     */
    public function save_items( $payload, $product_id, $saved_products ) {
        // Check if the product is already in the list of saved products.
        if( $saved_products && !empty( $saved_products ) ) {
            return [
                'success'  => false,
                'message'  => 'Product already added',
                'products' => [],
            ];
        }

        // Retrieve course details based on the product ID.
        $course         = Wpfnl_lms_learndash_functions::get_course_by_id( $product_id );
        $course_details = Wpfnl_lms_learndash_functions::get_course_details_by_id( $product_id );

        // Get the price of the course.
        $price = function_exists( 'learndash_get_course_price' ) ? learndash_get_course_price( $course ) : '';
        $price = isset( $price[ 'price' ] ) ? $price[ 'price' ] : $price;

        // Get the image ID and URL for the course.
        $image_id = get_post_meta( $product_id, '_thumbnail_id', true );
        $image    = wp_get_attachment_image_src( $image_id, 'single-post-thumbnail' );

        // Prepare and return product details.
        return [
            'success'  => true,
            'products' => [
                'id'                => $product_id,
                'title'             => $course->post_title,
                'price'             => $price,
                'trial_price'       => $course_details[ 'trial_price' ],
                'numeric_price'     => $price,
                'no_of_lession'     => $course_details[ 'no_of_lesson' ],
                'type'              => $course_details[ 'type' ],
                'currency'          => function_exists( 'learndash_get_currency_symbol' ) ?
                    Wpfnl_lms_learndash_functions::get_currency_symbol_from_country_code( learndash_get_currency_symbol() ) :
                    Wpfnl_lms_learndash_functions::get_currency_symbol_from_country_code( learndash_30_get_currency_symbol() ),
                'sale_price'        => $price,
                'regular_price'     => $price,
                'quantity'          => 1,
                'image'             => isset( $image[ 0 ] ) ? $image[ 0 ] : '',
                'discount_type'     => '',
                'discount_value'    => '',
                'product_edit_link' => get_edit_post_link( $product_id ),
                'product_view_link' => get_permalink( $product_id ),
            ],
        ];
    }

    /**
     * Change checkout column name in checkout tab 
     * 
     * @return array
     * @since 1.0.0 
     */
    public function change_checkout_columns_in_admin( $columns, $funnel_id ){

        if( $funnel_id ){
            $type = get_post_meta( $funnel_id, '_wpfnl_funnel_type', true );
            if( 'lms' === $type ){
                $columns = [];
                $columns = [
                    'product-name' 			=> 'Course',
                    'product-price' 		=> 'Lessons',
                    'product-quantity' 		=> 'Type',
                    'total-price' 			=> 'Price',
                    'product-action' 		=> 'Actions',
                ];
            }
        }
        return $columns;
    }

    /**
     * Change order bump position for lms
     * 
     * @param Array
     * @param String
     * 
     * @return Array
     */
    public function change_ob_positions_in_admin( $positions, $funnel_id ){
        if( $funnel_id ){
            $type = get_post_meta( $funnel_id, '_wpfnl_funnel_type', true );
            if( 'lms' === $type ){
                $positions = [];
                $positions = [
                    [
                        'name'  => 'Before product preview',
                        'value' => 'before-lms-order',
                    ],
                    [
                        'name'  => 'After product preview',
                        'value' => 'after-lms-order',
                    ],
                    [
                        'name'  => 'Pop-up offer',
                        'value' => 'popup',
                    ],
                ];
            }
        }
        return $positions;
    }

    /**
     * Get courses for checkout step
     * 
     * @param String $step_id
     * @return Array
     */
    public function get_items( $step_id = '' ){
        $response = [];
        
        if( $step_id ){
            $products 			=  get_post_meta($step_id, '_wpfnl_checkout_products', true);
            $discount 			=  get_post_meta($step_id, '_wpfnl_checkout_discount_main_product', true);
            if( $products && Wpfnl_lms_learndash_functions::is_learndash_active()){
                $course   = Wpfnl_lms_learndash_functions::get_course_by_id( $products[ 0 ][ 'id' ] );
                $course_details   = Wpfnl_lms_learndash_functions::get_course_details_by_id( $products[ 0 ][ 'id' ] );
                $price    = function_exists( 'learndash_get_course_price' ) ? learndash_get_course_price( $course ) : '';
                $price	  = isset( $price['price'] ) ? $price['price'] : $price;
                $image_id = get_post_meta( $products[ 0 ][ 'id' ], '_thumbnail_id', true );
                $image    = wp_get_attachment_image_src( $image_id, 'single-post-thumbnail' );

                $response['products'][] = array(
                    'id' 				=> $products[0]['id'],
                    'title' 			=> $course->post_title,
                    'price' 			=> $price,
                    'trial_price' 		=> $course_details['trial_price'],
                    'numeric_price'     => $price,
                    'no_of_lession'     => $course_details['no_of_lesson'],
                    'type'     			=> $course_details['type'],
                    'currency' 			=> function_exists( 'learndash_get_currency_symbol' ) ? Wpfnl_lms_learndash_functions::get_currency_symbol_from_country_code(learndash_get_currency_symbol()) : Wpfnl_lms_learndash_functions::get_currency_symbol_from_country_code(learndash_30_get_currency_symbol()),
                    'sale_price' 		=> $price,
                    'regular_price' 	=> $price,
                    'quantity' 			=> 1,
                    'image' 			=> isset( $image[0] ) ? $image[0] : '',
                    'discount_type' 	=> '',
                    'discount_value' 	=> '',
                    'product_edit_link' => get_edit_post_link($products[0]['id']),
                    'product_view_link' => get_permalink($products[0]['id']),
                );
                $response['coupon'] = '';
                $response['isMultipleProduct'] = '';
                $response['isQuantity'] = '';
                $response['discount'] = $discount;
            }else{
                $response['success'] = true;
                $response['products'] = [];
                $response['coupon'] = '';
                $response['isMultipleProduct'] = '';
                $response['isQuantity'] = '';
                $response['discount'] = $discount;
            }
            $response['columns'] = Wpfnl_functions::get_checkout_columns( $step_id );
        }
        return $response;
       
    }


    /**
     * Retrieve order bump item
     */
    public function retrieve_ob_item( $term = '' ){
        $formatted_courses = [];
        if( $term && Wpfnl_lms_learndash_functions::is_learndash_active() ){
            
            $courses = Wpfnl_lms_learndash_functions::search_course($term);
			foreach( $courses as $key=>$course ){
				$course_meta = learndash_get_course_meta_setting($course->ID);
				if( $course_meta['sfwd-courses_course_price_type'] === 'open' || $course_meta['sfwd-courses_course_price_type'] === 'closed'){
					continue;
				}
				$cr_meta = Wpfnl_lms_learndash_functions::get_course_details_by_id($course->ID);
				$price    = function_exists( 'learndash_get_course_price' ) ? learndash_get_course_price( $course ) : '';
				$price	  = isset( $price['price'] ) ? $price['price'] : $price;

				$image_id = get_post_meta($course->ID,'_thumbnail_id', true);
				$image = wp_get_attachment_image_src($image_id, 'single-post-thumbnail');
				$course_meta = learndash_get_course_meta_setting($course->ID);
				$formatted_courses[$course->ID] = [
					'name'       => $course->post_title,
					'price'      => $cr_meta['price'],
					'sale_price' => '',
					'html_price' => '',
					'title' => $course->post_title,
					'img' => array(
						'id' => $image_id ? $image_id : '',
						'url' => isset($image[0]) ? $image[0] : "",
					),
					'description' => $course_meta['sfwd-courses_course_materials']
				];
			}
        }
        return $formatted_courses;
    }
    
}
