<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://getwpfunnels.com/
 * @since      1.0.0
 *
 * @package    Wpfunnels_Pro_Lms
 * @subpackage Wpfunnels_Pro_Lms/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Wpfunnels_Pro_Lms
 * @subpackage Wpfunnels_Pro_Lms/includes
 * @author     WPFunnels <support@getwpfunnels.com>
 */
use WPFunnels\Lms\Settings\Wpfunnels_Lms_Settings;
use WPFunnels\Admin\FunnelType\Wpfunnels_Lms_Checkout;
use WPFunnels\lms\Shortcodes\Wpfnl_LMS_Shortcodes;
use WPFunnels\Modules\Frontend\LearnDash\Wpfnl_Frontend_LearnDash;
use WPFunnelsPro\Lms\Frontend\Modules\Webhook\Wpfnl_Pro_Lms_Webhook_Mapping;

use WPFunnelsPro\Lms\Widgets\Wpfnl_Pro_Lms_Widgets_Manager as Widget_Manager;
class Wpfunnels_Pro_Lms {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Wpfunnels_Pro_Lms_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;


	protected $widget_manager;
	protected $webhook;
	protected $lms_shortcode;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'WPFNL_PRO_LMS_VERSION' ) ) {
			$this->version = WPFNL_PRO_LMS_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'wpfunnels-pro-lms';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

		add_action( 'plugins_loaded', array( $this, 'init' ), 100 );
		add_action( 'wpfunnels/after_clear_transient', array($this, 'clear_lms_plugin_transients') );
        $this->init_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Wpfunnels_Pro_Lms_Loader. Orchestrates the hooks of the plugin.
	 * - Wpfunnels_Pro_Lms_i18n. Defines internationalization functionality.
	 * - Wpfunnels_Pro_Lms_Admin. Defines all hooks for the admin area.
	 * - Wpfunnels_Pro_Lms_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
         * The class responsible for auto loading all files of the
         * core plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'vendor/autoload.php';

		
		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/utils/class-wpfunnels-pro-lms-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/utils/class-wpfunnels-pro-lms-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wpfunnels-pro-lms-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-wpfunnels-pro-lms-public.php';

		$this->loader = new Wpfunnels_Pro_Lms_Loader();

	}


	/**
     * Initialize the core of the plugin
     *
     * @since 1.0.0
     */
    public function init()
    {
		if ( ! function_exists( 'get_plugins' ) || ! function_exists( 'is_plugin_active' ) ) {
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}
		
		$installed_plugins = get_plugins();
		/** check free plugin is activated or not */
		if( is_plugin_active( 'wpfunnels/wpfnl.php' ) ){
			
			$this->widget_manager  = Widget_Manager::getInstance()->init();
		}
        $this->lms_shortcode       = Wpfnl_LMS_Shortcodes::getInstance()->init();
		$this->webhook         = new Wpfnl_Pro_Lms_Webhook_Mapping();
	}

    /**
     * LMS shortcode class
     */
    public function init_hooks() {
        $shortcode = new Wpfnl_LMS_Shortcodes();
        add_action( 'init', array( $shortcode, 'init') );
    }

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Wpfunnels_Pro_Lms_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Wpfunnels_Pro_Lms_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin 			= new Wpfunnels_Pro_Lms_Admin( $this->get_plugin_name(), $this->get_version() );

		$plugin_admin_ajax 		= new Wpfunnels_Lms_Settings();
		$plugin_admin_checkout 	= new Wpfunnels_Lms_Checkout();
		
		$this->loader->add_action( 'admin_init', $plugin_admin_ajax, 'init_ajax');
		$this->loader->add_action( 'admin_init', $plugin_admin, 'create_lms_order_table');
		
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_filter( 'wpfunnels/is_enable_lms_settings', $plugin_admin, 'enable_lms_settings' );
		$this->loader->add_filter( 'wpfunnels/gutenberg_block_types', $plugin_admin, 'include_lms_block_type', 10, 1 );
		$this->loader->add_filter( 'wpfunnels/template_data', $plugin_admin, 'get_template_data', 10, 1 );
		$this->loader->add_filter( 'wpfunnels/addon-lists', $plugin_admin , 'register_addon_for_licensing', 10, 1 );
		$this->loader->add_filter( 'wpfunnels/modify_template_type', $plugin_admin , 'modify_template_type', 10, 1 );
		$this->loader->add_filter( 'wpfunnels/check_update_transient', $plugin_admin , 'check_update_transient' );
		
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Wpfunnels_Pro_Lms_Public( $this->get_plugin_name(), $this->get_version() );
		new Wpfnl_Frontend_LearnDash();
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Wpfunnels_Pro_Lms_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}


	/**
     * clear lms transient data
     *
     * @since 1.0.4
     */
    public function clear_lms_plugin_transients() {
        delete_site_transient( 'update_plugins' );
        $this->clear_lms_update_check_transient();
    }


	/**
     * clear lms update check transient data
     *
     * @since 1.0.4
     */
    private function clear_lms_update_check_transient() {
        global $wpdb;
        $sql   = "DELETE a, b FROM $wpdb->options a, $wpdb->options b
		WHERE a.option_name LIKE %s
		AND a.option_name NOT LIKE %s
		AND b.option_name = CONCAT( '_site_transient_timeout_', SUBSTRING( a.option_name, 17 ) )";
        $wpdb->query( $wpdb->prepare( $sql, $wpdb->esc_like( '_site_transient_wpfunnelspro-lms-check_for_plugin_update' ) . '%', $wpdb->esc_like( '_site_transient_timeout_' ) . '%' ) );
    }

}
