<?php
namespace WPFunnels\FunnelType;

use LearnDash\Core\Models\Product;
use WPFunnels\Modules\Frontend\LearnDash\Wpfnl_Frontend_LearnDash;
use WPFunnels\Wpfnl_functions;
use WPFunnels\FunnelType\Wpfnl_Public_Funnel_Type;
use WPFunnels\lms\helper\Wpfnl_lms_learndash_functions;

class Wpfnl_Public_Lms extends Wpfnl_Public_Funnel_Type
{

	/**
	 * Trigger an order bump action based on the checker condition.
	 *
	 * This function handles order bump triggers by determining whether the action should be 'accepted' or 'rejected'
	 * based on the value of the checker parameter. It then calls the corresponding order bump action function.
	 *
	 * @param int    $step_id   The ID of the step in the funnel.
	 * @param int    $product_id   The ID of the product associated with the order bump.
	 * @param int    $quantity  The quantity of the product.
	 * @param string $key       A unique key associated with the order bump.
	 * @param int    $user_id   The ID of the user making the order.
	 * @param int    $funnel_id The ID of the funnel.
	 * @param bool   $checker   The checker condition ('true' for acceptance, 'false' for rejection).
	 *
	 * @return mixed The result of the order bump action.
	 * @since 1.0.11
	 */
	public function wpfnl_order_bump_trigger( $step_id, $product_id, $quantity, $key, $user_id, $funnel_id, $checker ) {
		// Determine the action based on the checker condition.
		$action = 'true' === (string) $checker ? 'accepted' : 'rejected';

		// Call the corresponding order bump action function with the determined action.
		return $this->wpfnl_order_bump_action( $step_id, $product_id, $user_id, $funnel_id, $action );
	}


	/**
	 * Perform an order bump action based on the specified action type.
	 *
	 * This function handles order bump actions, either 'accepted' or 'rejected', based on the provided action type.
	 * It retrieves course details, renders product design, and generates payment nonces as needed.
	 *
	 * @param int    $step_id   The ID of the step in the funnel.
	 * @param int    $product_id   The ID of the product associated with the order bump.
	 * @param int    $user_id   The ID of the user making the order.
	 * @param int    $funnel_id The ID of the funnel.
	 * @param string $action    The action type ('accepted' or 'rejected') to perform.
	 *
	 * @return array An array containing the result of the order bump action and related data.
	 * @since 1.0.11
	 */
	private function wpfnl_order_bump_action( $step_id, $product_id, $user_id, $funnel_id, $action ) {
		// Check if LearnDash is active; if not, return an empty array.
		if ( ! Wpfnl_lms_learndash_functions::is_learndash_active() ) {
			return [];
		}

		// Prepare a success message based on the action type.
		$message = __( 'Order bump product has been successfully ', 'wpfnl' );

		// Get course details for the specified product.
		if( 'accepted' === $action ) {
			setcookie( 'wpfnl_ld_order_bump_accepted', 'true' );
			$course = Wpfnl_lms_learndash_functions::get_course_details_by_id( $product_id );
			$message .=  __( 'added.', 'wpfnl' );
		}
		else {
			if( isset( $_COOKIE[ 'wpfnl_ld_order_bump_accepted' ] ) ) {
				unset( $_COOKIE[ 'wpfnl_ld_order_bump_accepted' ] );
				setcookie( 'wpfnl_ld_order_bump_accepted', '', -1, '/' );
			}
			$course = Wpfnl_lms_learndash_functions::get_course_details( $step_id );
			$message .=  __( 'removed.', 'wpfnl' );
		}

		// Render the main product design HTML based on the course details and step ID.
		$html = $this->render_main_product_design( $course, $step_id, $funnel_id );

		/**
		 * Trigger a custom action hook based on the provided action type.
		 *
		 * @param int $step_id    The ID of the order bump step.
		 * @param int $product_id The ID of the product that was accepted as an order bump.
		 *
		 * @since 1.0.11
		 */
		do_action( "wpfunnels/order_bump_{$action}", $step_id, $product_id );

		return [
			'status'       => 'success',
			'message'      => $message,
			'order_bump'   => true,
			'html'         => $html,
			'course'       => $course,
			'user_id'      => $user_id,
			'step_id'      => $step_id
		];
	}


    /**
	 * render main product design
	 */
	private function render_main_product_design(  $course , $step_id, $funnel_id ){

		$course_title 		= $course['title'];
		$description 		= $course['description'];
		$no_of_lesson 		= $course['no_of_lesson'];
		$course_type 		= $course['type'];
		$billing_cycle 		= $course['billing_cycle'];
		$recurring_time 	= $course['recurring_time'];
		$price 				= $course['price'];
		$trial_price 		= $course['trial_price'];
		$trial_period 		= $course['trial_period'];
		$is_expire 			= $course['is_expire'];
		$expire_days 		= $course['expire_days'];
		$image 				= $course['image'];
		$billing_cycle_unit = $course['billing_cycle_unit'];
		$trial_period_unit  = $course['trial_period_unit'];
		$currency  = $course['currency'];
		$discount_price  	= isset($course['discount_price']) ? $course['discount_price'] : '';

		
		//----course type------
		if( $course_type == 'subscribe' ) {
			$course_type_text = 'Subscription';

		}else if( $course_type == 'open' || $course_type == 'free' ){
			$course_type_text = 'Free';

		}else if( $course_type == 'paynow' ){
			$course_type_text = 'One-time Payment';

		}else {
			$course_type_text = 'Closed';
		}

		//----billing cycle unit------
		if( $course['billing_cycle_unit'] == 'Y' ) {
			$billing_cycle_unit_text = 'year';

		}else if( $course['billing_cycle_unit'] == 'M' ){
			$billing_cycle_unit_text = 'month';

		}else if( $course['billing_cycle_unit'] == 'W' ){
			$billing_cycle_unit_text = 'week';

		}else {
			$billing_cycle_unit_text = 'day';
		}

		//----trial period unit------
		if( $course['trial_period_unit'] == 'Y' ) {
			$trial_period_unit_text = 'year';

		}else if( $course['trial_period_unit'] == 'M' ){
			$trial_period_unit_text = 'month';

		}else if( $course['trial_period_unit'] == 'W' ){
			$trial_period_unit_text = 'week';

		}else {
			$trial_period_unit_text = 'day';
		}
		ob_start();
		do_action( 'wpfunnels/before_checkout_form', $step_id );
		?>
		<div class="lms-checkout-box">
			<div class="lms-checkout-header">
				<h4>Order Details</h4>
			</div>
			<div class="lms-checkout-body">
				<div class="course-image">
					<img src="<?php echo $image; ?>" alt="course image" />
				</div>

				<div class="course-content">
					<h2 class="course-title">
						<?php echo $course_title ?>
						<span class="no-of-lesson">- <?php echo $no_of_lesson ?> Lessons</span>
					</h2>

					<div class="lms-single-block course-details-block">
						<h4 class="lms-block-title">Course Details</h4>
						<div class="course-description">
							<?php echo $description ?>
						</div>
					</div>

					<div class="lms-single-block course-plan-block">
						<h4 class="lms-block-title">Plan Details</h4>
						<div class="course-plan">
							<?php
								if( $course_type == 'subscribe' ){
									echo $course_type_text .' - <span class="primary-color">'.$price.'/'.$billing_cycle_unit_text.'</span>';

									if( !empty( $trial_period ) ){
										echo '<small> ('.$trial_period.' '.$trial_period_unit_text.' trial period)</small>';
									}

								} else if( $course_type == 'paynow' ){
									
									if( $discount_price ){
										echo $course_type_text .' - <span class="primary-color"><del>'.$currency.$price.'</del>  <ins style="text-decoration:none">'.$currency.$discount_price.'</ins></span>';
									}else{
										echo $course_type_text .' - <span class="primary-color">'.$currency.$price.'</span>';
									}
									if( $is_expire == 'on' ){
										echo '<small> (expires in '.$expire_days.' days)</small>';
									}
								}else{
									echo $course_type_text;

									if( $is_expire == 'on' ){
										echo '<small> (Login or register to enroll) (expires in '.$expire_days.' days)</small>';
									}
								}
							?>
						</div>
					</div>
				</div>
			</div>
		</div>
		<div class="lms-checkout-footer">
			<?php
			if( $course_type == 'subscribe' ){
				if( !empty( $trial_period ) ){
					?>

					<p class="footer-title">Course Trial Fees <span class="footer-price"><?php echo $currency.$trial_price; ?></span></p>

				<?php }else{ ?>

					<p class="footer-title">Course Subscription Fees <span class="footer-price"><?php echo $currency.$price; ?></span></p>
					<?php
				}
			}else if( $course_type == 'paynow' ){
				if( $discount_price ){ ?>
					<p class="footer-title">Course Fees <span class="footer-price"><del><?php echo $currency.$price; ?></del> <ins style="text-decoration:none"> <?php echo $currency.$discount_price; ?> </ins></span></p>
				<?php } else{ ?>
					<p class="footer-title">Course Fees <span class="footer-price"><?php echo $currency.$price; ?></span></p>
				<?php }
				?>
			<?php }else{ ?>
				<p class="footer-title">Course Fees <span class="footer-price">Free</span></p>
			<?php } ?>

			<!-- buy now button -->
			<?php
			if (is_user_logged_in() ){
				$course_access   = sfwd_lms_has_access( $course[ 'id' ], get_current_user_id() );
				$next_step_url   = Wpfnl_lms_learndash_functions::get_next_step_url( $funnel_id, $step_id ) . '?wpfnl_ld_payment=free&wpfnl_course_status=enrolled';
				$lms_button_text = get_option( 'learndash_settings_custom_labels' );
				$button_text     = !empty( $lms_button_text[ 'button_take_this_course' ] ) ? $lms_button_text[ 'button_take_this_course' ] : __( 'Take This Course', 'wpfunnels-pro-lms' );
				if ($course_access ){
					echo '<a class="btn-default" href="'.$next_step_url.'" id="wpfnl-lms-access-course">'.$button_text.'</a>';
					echo '<span class="wpfnl-lms-access-course-message"></span>';
				}else if($course_type == 'free'){
					$next_step_url = Wpfnl_lms_learndash_functions::get_next_step_url($funnel_id,$step_id);
					echo '<a class="btn-default" href="'.$next_step_url.'" data-user_id="'.get_current_user_id().'" data-step_id="'.$step_id.'" data-course_id="'.$course['id'].'" id="wpfnl-lms-free-course">'.$button_text.'</a>';
					echo '<span class="wpfnl-lms-free-course-message"></span>';
				}
				else{
					echo do_shortcode( "[learndash_payment_buttons course_id='{$course['id']}']" );
				}
			}else{
				echo do_shortcode('[learndash_login]');
			}
			?>
		</div>
		<?php
	
		return ob_get_clean();
	}
}