<?php


namespace WPFunnels\Modules\Frontend\LearnDash;

use WPFunnels\Wpfnl_functions;
use WPFunnels\lms\helper\Wpfnl_lms_learndash_functions;

class Wpfnl_Frontend_LearnDash
{
	public $funnel_id;

	public $step_id;

	// const CHECKOUT = "";

	/**
	 * Constructor function for a class.
	 *
	 * This constructor sets up various filters and actions for LearnDash-related functionality.
	 * It defines validation rules, modifies payment buttons, handles redirects, and adds AJAX actions.
	 * 
	 * @since 1.0.11
	 */
	public function __construct()
	{
		// Define validation rules.
		$this->validations = [
			'logged_in' => true,
			'user_can' => 'manage_options',
		];

		// Add filters for modifying payment buttons and handling redirects.
		add_filter('learndash_payment_button_markup', [$this, 'modify_payment_buttons'], 99);
		add_filter('learndash_payment_option_url_success', [$this, 'redirect_to_next_step'], 99);
		add_filter('learndash_stripe_purchase_success_url', [$this, 'redirect_to_next_step'], 99);
		add_action( 'learndash_registration_form', [$this, 'add_step_id_in_learndash_registration_form'], 99 );
        add_filter( 'login_form_top', [ $this, 'add_step_id_in_learndash_login_form' ], 99 );
        add_filter( 'wp_redirect', [ $this, 'redirect_to_wpfnl_step_after_user_registration' ], 99 );

		// Add AJAX actions for specific functionality.
		add_action('wp_ajax_wpfnl_learndash_already_enroll_course', [$this, 'wpfnl_learndash_already_enroll_course']);
		add_action('wp_ajax_nopriv_wpfnl_learndash_already_enroll_course', [$this, 'wpfnl_learndash_already_enroll_course']);

		add_action('wp_ajax_wpfnl_learndash_free_course', [$this, 'wpfnl_learndash_free_course']);
		add_action('wp_ajax_nopriv_wpfnl_learndash_free_course', [$this, 'wpfnl_learndash_free_course']);
		add_action('learndash_login_model_register_url', [$this, 'wpfnl_learndash_login_model_register_url']);

		add_action('wp_ajax_wpfnl_get_product_info', [$this, 'wpfnl_get_product_info']);
		add_action('wp_ajax_nopriv_wpfnl_get_product_info', [$this, 'wpfnl_get_product_info']);

        add_filter('learndash_get_price_by_coupon', [$this, 'wpfnl_learndash_get_price_by_coupon']);
	}

    /**
     * Add the 'wpfnl_step_id' as a hidden input field in the LearnDash registration form.
     *
     * This function adds a hidden input field to the LearnDash registration form with the name 'wpfnl_step_id'
     * and the value based on the 'wpfnl_step_id' query parameter if it is associated with a WP Funnel step.
     *
     * @since 1.0.11
     */
    public function add_step_id_in_learndash_registration_form() {
        // Get the 'wpfnl_step_id' query parameter and sanitize it as an integer.
        $step_id = filter_input( INPUT_GET, 'wpfnl_step_id', FILTER_SANITIZE_NUMBER_INT );
        // Check if the provided 'wpfnl_step_id' is associated with a WP Funnel step.
        if( self::maybe_wpfnl_step( $step_id ) ) {
            // Add a hidden input field with 'wpfnl_step_id' as the name and the sanitized value as the value.
            echo "<input type='hidden' name='wpfnl_step_id' value='{$step_id}'>";
        }
    }

    /**
     * Add a hidden input field with 'wpfnl_step_id' to the LearnDash login form.
     *
     * This function modifies the content of the LearnDash login form by adding a hidden input field.
     * The hidden input field contains the 'wpfnl_step_id' query parameter value, which is sanitized as an integer.
     * This is done if the 'wpfnl_step_id' is associated with a WP Funnel step, making it accessible in the form data.
     *
     * @param string $content The original content of the LearnDash login form.
     *
     * @return string The modified content of the LearnDash login form with the hidden input field.
     * @since 1.0.11
     */
    public function add_step_id_in_learndash_login_form( $content ) {
        // Get the 'wpfnl_step_id' query parameter from the GET data and sanitize it as an integer.
        $step_id = filter_input( INPUT_GET, 'wpfnl_step_id', FILTER_SANITIZE_NUMBER_INT );

        // Check if the provided 'wpfnl_step_id' is associated with a WP Funnel step.
        if( self::maybe_wpfnl_step( $step_id ) ) {
            // Add a hidden input field with 'wpfnl_step_id' as the name and the sanitized value as the value.
            return $content . "<input type='hidden' name='wpfnl_step_id' value='{$step_id}'>";
        }

        // If 'wpfnl_step_id' is not associated with a WP Funnel step, return the original content.
        return $content;
    }

    /**
     * Redirect users to a WPFunnel step page after user registration.
     *
     * This function checks if the 'wpfnl_step_id' query parameter is associated with a WP Funnel step.
     * If it is, it redirects the user to the corresponding WP Funnel step page; otherwise, it maintains the original redirect URL.
     *
     * @param string $redirect_url The original redirect URL after user registration.
     *
     * @return string The modified redirect URL based on the 'wpfnl_step_id' query parameter.
     * @since 1.0.11
     */
    public function redirect_to_wpfnl_step_after_user_registration( $redirect_url ) {
        // Get the 'wpfnl_step_id' query parameter from the POST data and sanitize it as an integer.
        $step_id = filter_input( INPUT_POST, 'wpfnl_step_id', FILTER_SANITIZE_NUMBER_INT );
        $action  = filter_input( INPUT_POST, 'wp-submit', FILTER_SANITIZE_STRING );

        // Check if the provided 'wpfnl_step_id' is associated with a WP Funnel step.
        if( self::maybe_wpfnl_step( $step_id ) && ( 'Register' === $action || 'Log In' === $action ) ) {
            // Return the permalink of the WP Funnel step page for redirection.
            return get_permalink( $step_id );
        }

        // If 'wpfnl_step_id' is not associated with a WP Funnel step, maintain the original redirect URL.
        return $redirect_url;
    }

    /** Modify the PayPal payment button for LearnDash courses.
     *
     * This function modifies the PayPal payment button to include custom return URLs with
     * additional parameters for tracking purposes. It checks if PayPal is enabled and if the
     * necessary classes exist before making modifications.
     *
     * @param string $button Course payment button.
     *
     * @return array An updated array of PayPal payment buttons with modified return URLs.
	 * @since 1.0.11
     */
    public function modify_payment_buttons( $button ) {
        $paypal_settings = get_option( 'learndash_settings_paypal', [] );
        $step_id   = self::get_wpfnl_step_id();

        if( !empty( $step_id ) ) {
            if( !empty( $paypal_settings[ 'enabled' ] ) && 'on' === $paypal_settings[ 'enabled' ] ) {
                $course_id = $this->get_step_course_id( $step_id );

                $site_url                  = get_home_url( null, '', 'http' );
                $paypal_return_url_search  = "<input type=\"hidden\" name=\"return\" value=\"{$site_url}?";
                $paypal_return_url_replace = "<input type=\"hidden\" name=\"return\" value=\"{$site_url}?wpfnl_step_id={$step_id}&ld_register_id={$course_id}&";
                $button                    = str_replace( $paypal_return_url_search, $paypal_return_url_replace, $button );

                $site_url                  = get_home_url( null, '', 'https' );
                $paypal_return_url_search  = "<input type=\"hidden\" name=\"return\" value=\"{$site_url}?";
                $paypal_return_url_replace = "<input type=\"hidden\" name=\"return\" value=\"{$site_url}?wpfnl_step_id={$step_id}&ld_register_id={$course_id}&";
                $button                    = str_replace( $paypal_return_url_search, $paypal_return_url_replace, $button );
            }
            $user_id = get_current_user_id();
            $button = str_replace( '</form>', "<input type='hidden' name='wpfnl_user_id' value='{$user_id}'></form>", $button );
            $button = str_replace( '</form>', "<input type='hidden' name='wpfnl_step_id' value='{$step_id}'></form>", $button );
        }
        return $button;
    }

    /**
     * Get the course ID associated with a specific funnel step.
     *
     * This function retrieves the course ID from various sources based on the given step ID.
     * It checks for different scenarios, including AJAX requests for order bump, query parameters, and post metadata.
     *
     * @param int $step_id The ID of the funnel step.
     *
     * @return int|null The course ID if found, or null if not found or invalid.
     * @since 1.0.11
     */
    private function get_step_course_id( $step_id ) {
        // Check if the function is called during an AJAX request for 'wpfnl_order_bump_ajax'.
        if(
            'wpfnl_order_bump_ajax' === filter_input( INPUT_POST, 'action' ) &&
            filter_input( INPUT_POST, 'checker', FILTER_VALIDATE_BOOL )
        ) {
            $course_id = filter_input( INPUT_POST, 'product', FILTER_SANITIZE_NUMBER_INT );
            if( 'sfwd-courses' === get_post_type( $course_id ) ) {
                return $course_id;
            }
        }

        // Check for the 'ld_register_id' query parameter.
        $course_id = filter_input( INPUT_GET, 'ld_register_id', FILTER_SANITIZE_NUMBER_INT );
        if( 'sfwd-courses' === get_post_type( $course_id ) ) {
            return $course_id;
        }

        // Check for the 'ld_register_id' post parameter.
        $course_id = filter_input( INPUT_POST, 'ld_register_id', FILTER_SANITIZE_NUMBER_INT );
        if( 'sfwd-courses' === get_post_type( $course_id ) ) {
            return $course_id;
        }

        // Check if the funnel step is associated with a course in its post metadata.
        $step_type = get_post_meta( $step_id, '_step_type', true );
        $product   = get_post_meta( $step_id, "_wpfnl_{$step_type}_products", true );
        if( !empty( $product[ 0 ][ 'id' ] ) && 'sfwd-courses' === get_post_type( $product[ 0 ][ 'id' ] ) ) {
            return $product[ 0 ][ 'id' ];
        }

        return null; // Return null if no valid course ID is found.
    }

    /**
     * Redirect the user to the next step in a funnel after payment.
     *
     * This function determines the user's payment method (PayPal or Stripe), extracts user and course information from input data,
     * updates the learning management system (LMS) order, and redirects the user to the next step in the funnel.
     *
     * @param string $return_url The default return URL.
     *
     * @return string The URL to which the user should be redirected after payment.
     * @since 1.0.11
     */
    public function redirect_to_next_step( $return_url ) {
        $payment_method = 'paypal';
        $user_id        = filter_input( INPUT_POST, 'custom', FILTER_SANITIZE_STRING );

        // Check if 'custom' parameter is empty, indicating Stripe payment.
        if( empty( $user_id ) ) {
            $user_id        = filter_input( INPUT_POST, 'stripe_user_id', FILTER_SANITIZE_NUMBER_INT );
            $payment_method = $user_id ? 'stripe' : $payment_method;
        }

        $step_id   = self::get_wpfnl_step_id();
        $course_id = $this->get_step_course_id( $step_id );
        $funnel_id = $step_id ? get_post_meta( $step_id, '_funnel_id', true ) : null;

        if( $funnel_id && $course_id ) {
            // Update the LMS order with funnel, step, user, course, and payment method information.
            $this->update_lms_order( $funnel_id, $step_id, $user_id, $course_id, $payment_method );

            // Get the URL of the next step in the funnel.
            return Wpfnl_lms_learndash_functions::get_next_step_url( $funnel_id, $step_id );
        }

        return $return_url;
    }


    /**
     * Update a LearnDash LMS order based on funnel and course details.
     *
     * This function is responsible for updating a LearnDash LMS order with information
     * from a funnel, step, and user. It retrieves the discounted price of the course,
     * course details, and other relevant information, then calls the function to create
     * a new LearnDash LMS order.
     *
     * @param int    $funnel_id        The ID of the funnel associated with the order.
     * @param int    $step_id          The ID of the funnel step associated with the order.
     * @param int    $user_id          The ID of the user who made the purchase.
     * @param int    $course_id        The ID of the LearnDash course in the order.
     * @param string $payment_method   The payment method used for the order.
	 * 
	 * @since 1.0.11
     */
    private function update_lms_order( $funnel_id, $step_id, $user_id, $course_id, $payment_method ) {
        $discounted_price = Wpfnl_lms_learndash_functions::get_discounted_course_price( $step_id );
        $order_details = [
            'step_id' 			=> $step_id,
            'funnel_id' 		=> $funnel_id,
            'user_id' 			=> $user_id,
            'course_details' 	=> Wpfnl_lms_learndash_functions::get_course_details_by_id($course_id),
            'discounted_price' 	=> $discounted_price,
        ];
        $this->create_lms_order( $order_details , $payment_method );
    }

	/**
	 * Creates a new LMS (Learning Management System) order record in the database.
	 *
	 * @param array $order_details An array containing order details.
	 * @param string $payment_method The payment method used for the order.
	 * 
	 * @since 1.0.11
	 */
	private function create_lms_order( $order_details = [], $payment_method = '' ){
		global $wpdb;
		$wpfnl_lms_order_table = $wpdb->prefix . 'wpfnl_lms_order';
		$funnel_id             = $order_details[ 'funnel_id' ];
		$step_id               = $order_details[ 'step_id' ];
		$user_id               = $order_details[ 'user_id' ];
		$course_id             = $order_details[ 'course_details' ]['id'];

		$query = $wpdb->prepare( "SELECT * FROM {$wpfnl_lms_order_table} WHERE funnel_id = %d AND step_id = %d AND user_id = %d", $funnel_id, $step_id, $user_id );
		$result = $wpdb->get_results( $query );

		if( !empty($result) ){
			$query = $wpdb->prepare("UPDATE {$wpfnl_lms_order_table} SET meta='order' WHERE funnel_id = %d AND step_id = %d AND user_id = %d", $funnel_id, $step_id, $user_id);
			$wpdb->query( $query );
		}

		$step_type= get_post_meta( $step_id, '_step_type', true );

		if( 'checkout' === $step_type ){
			$main_product = get_post_meta($step_id,'_wpfnl_checkout_products',true);
			$key = array_search($course_id, array_column($main_product, 'id'));
			if( false !== $key ){
				$step_type = 'main_product';
			}else{
				$step_type = 'orderbump';
			}
		}

        if( !empty( $order_details[ 'course_details' ][ 'type' ] ) && 'subscribe' === $order_details[ 'course_details' ][ 'type' ] && !empty( $order_details[ 'course_details' ][ 'trial_price' ] ) ) {
            $amount = $order_details[ 'course_details' ][ 'trial_price' ];
        }
        elseif( 'orderbump' !== $step_type && !empty( $order_details[ 'discounted_price' ] ) ) {
            $amount = $order_details[ 'discounted_price' ];
        }
        else {
            $amount = $order_details[ 'course_details' ][ 'price' ] ?? 0;
        }

		$order_data = array(
			'funnel_id'    => $order_details[ 'funnel_id' ],
			'step_id'      => $order_details[ 'step_id' ],
			'user_id'      => $order_details[ 'user_id' ],
			'meta'         => 'new_order',
			'payment_method'=> $payment_method,
			'course_id'		=> $course_id,
			'amount'		=> $amount,
			'step_type'		=> $step_type,
			'order_data'   => serialize( $order_details[ 'course_details' ] ),
			'date_created' => date( "Y-m-d h:i:s" )
		);

		$wpdb->insert(
			$wpfnl_lms_order_table,
			$order_data
		);
		$order_id = $wpdb->insert_id;
		
		$this->set_automation_cookie($order_details[ 'funnel_id' ], $order_details[ 'step_id' ], $step_type, $order_id );
	}

    /**
     * Get the ID of the current 'wpfunnel_steps' post.
     *
     * This function attempts to retrieve the ID of the current 'wpfunnel_steps' post in various ways:
     * 1. If called within the context of a WordPress post loop, it gets the current post's ID.
     * 2. It checks if the 'wpfnl_step_id' is present in the URL query parameters.
     * 3. If the function is called during an AJAX request for 'wpfnl_order_bump_ajax', it looks for the 'step_id' parameter.
     *
     * @return int|null The ID of the 'wpfunnel_steps' post, or null if not found.
     */
    public static function get_wpfnl_step_id() {
        // Attempt to get the current post's ID.
        $step_id = get_the_ID();

        // Check if the current post belongs to the 'wpfunnel_steps' custom post type.
        if( self::maybe_wpfnl_step( $step_id ) ) {
            return $step_id;
        }

        // Check if 'wpfnl_step_id' is present in the URL query parameters.
        $step_id = filter_input( INPUT_GET, 'wpfnl_step_id', FILTER_SANITIZE_NUMBER_INT );
        if( self::maybe_wpfnl_step( $step_id ) ) {
            return $step_id;
        }

        // Check if 'wpfnl_step_id' is present in the URL query parameters.
        $step_id = filter_input( INPUT_POST, 'wpfnl_step_id', FILTER_SANITIZE_NUMBER_INT );
        if( self::maybe_wpfnl_step( $step_id ) ) {
            return $step_id;
        }

        // Check if the function is called during an AJAX request for 'wpfnl_order_bump_ajax'.
        if( 'wpfnl_order_bump_ajax' === filter_input( INPUT_POST, 'action' ) ) {
            $step_id = filter_input( INPUT_POST, 'step_id', FILTER_SANITIZE_NUMBER_INT );
            if( self::maybe_wpfnl_step( $step_id ) ) {
                return $step_id;
            }
        }

        // Return null if no valid 'wpfunnel_steps' post ID is found.
        return null;
    }

    /**
     * Check if a post is of the 'wpfunnel_steps' custom post type.
     *
     * This function checks whether a post with the given ID belongs to the 'wpfunnel_steps' custom post type.
     *
     * @param int $post_id The ID of the post to check.
     *
     * @return bool True if the post belongs to the 'wpfunnel_steps' post type, false otherwise.
     */
    public static function maybe_wpfnl_step( $post_id ) {
        // Check if the post with the given ID belongs to the 'wpfunnel_steps' custom post type.
        return 'wpfunnel_steps' === get_post_type( $post_id );
    }

    /**
	 * Set automation cookie for lms
	 */
	private function set_automation_cookie( $funnel_id = '', $step_id = '', $step_type = '', $order_id = '' ){
		$cookie_name        = 'wpfunnels_automation_data';
        $cookie             = isset( $_COOKIE[$cookie_name] ) ? json_decode( wp_unslash( $_COOKIE[$cookie_name] ), true ) : array();
		if( $step_type ){

			if( 'main_product' === $step_type ){
				if( $order_id ){
					$cookie['main_order_id']    = $order_id;
            		$cookie['funnel_id']        = $funnel_id;
				}
			}elseif( 'upsell' === $step_type || 'downsell' === $step_type ){
				$offer_settings = \WPFunnels\Wpfnl_functions::get_offer_settings();
				$cookie['child_order']   = $offer_settings['offer_orders'] == 'child-order' ? true : false;
				$cookie['offer'][$step_id]   = array(
					'id'            => $step_id,
					'type'          => $step_type,
					'order_id'      => $order_id,
					'status'        => 'accepted'
				);

			}
			if( isset($cookie['ob_accepetd_products']) && !empty($cookie['ob_accepetd_products']) ){
				$cookie['main_order_id']    = false;
			}
			if( isset($cookie['ob_accepetd_products']) && empty($cookie['ob_accepetd_products']) ){
				$cookie['orderbump_accepted']    = false;
			}
			if( !isset($cookie['already_enrolled']) ){
				$cookie['already_enrolled']   = [];
			}
			if(!isset($cookie['funnel_id'])) {
				$cookie['funnel_id']   = $funnel_id;
			}
			do_action('wpfunnels/lms_course_enroll', $cookie , $step_id);
			setcookie( $cookie_name, wp_json_encode( $cookie ), strtotime( '+1 days' ), '/', COOKIE_DOMAIN );
		}
	}

	/**
	 * Update lms order meta when user is already enrolled
	 *
	 *
	 */
	public function wpfnl_learndash_already_enroll_course(){
		$response = [
			'success' => false,
		];
		
		if( isset($_POST['step_id']) && isset($_POST['user_id']) ){
			$funnel_id = get_post_meta($_POST['step_id'],'_funnel_id',true);
			$user_id   = $_POST['user_id'];
			$step_id   = $_POST['step_id'];

			$step_type = '';
			if( $funnel_id && $user_id && $step_id ){
				Wpfnl_lms_learndash_functions::update_lms_order_status( $funnel_id, $step_id, $user_id );
				$step_type = get_post_meta($step_id,'_step_type',true);
			}
			$response = [
				'success' => true,
			];

			if( $step_type ){
				/** Set Cookie Data */
				$cookie_name        = 'wpfunnels_automation_data';
				$cookie             = isset( $_COOKIE[$cookie_name] ) ? json_decode( wp_unslash( $_COOKIE[$cookie_name] ), true ) : array();
				
				if( isset($cookie['already_enrolled']) ){
					array_push($cookie['already_enrolled'],$step_type);
				}else{
					$cookie['already_enrolled']   = [];
					array_push($cookie['already_enrolled'],$step_type);
				}

				if( 'upsell' === $step_type || 'downsell' === $step_type ){
					$cookie['offer'][$step_id]   = array(
						'id'            => $step_id,
						'type'          => $step_type,
						'status'        => 'accepted'
					);
				}

				if( !isset($cookie['user_info']) ){
					$current_user = wp_get_current_user();
					$cookie['user_info']   = [
						'id'            => get_current_user_id(),
						'first_name'    => $current_user->user_firstname,
						'last_name'     => $current_user->user_lastname,
						'email'         => $current_user->user_email
					];
				}
				if( !isset($cookie['ob_accepetd_products']) ){
					$cookie['ob_accepetd_products'] = [];
				}

				if(!isset($cookie['orderbump_accepted'])){

					if( !empty($cookie['ob_accepetd_products']) ){
						$cookie['orderbump_accepted']   = true;
					}else{
						$cookie['orderbump_accepted']   = false;
					}
				}
				if(!isset($cookie['funnel_id'])) {
					$cookie['funnel_id']   = $funnel_id;
				}
				setcookie( $cookie_name, wp_json_encode( $cookie ), strtotime( '+1 days' ), '/', COOKIE_DOMAIN );
			}

		}
		do_action('wpfunnels/lms_course_enroll', $cookie , $step_id);
		wp_send_json( $response );
	}


	/**
	 * Update lms order meta when user is already enrolled
	 *
	 *
	 */
	public function wpfnl_learndash_free_course() {
		$response = [ 'success' => false ];
		$posts    = filter_input_array( INPUT_POST );

		if( isset( $posts[ 'step_id' ], $posts[ 'user_id' ] ) ) {
			$post_data = [];
			$postData  = $posts[ 'data' ] ?? '';
			wp_parse_str( $postData, $post_data );
			$funnel_id = get_post_meta( $posts[ 'step_id' ], '_funnel_id', true );
			$user_id   = $posts[ 'user_id' ];
			$step_id   = $posts[ 'step_id' ];
			if( isset( $posts[ 'course_id' ] ) ) {
				$courses   = learndash_user_get_enrolled_courses( $user_id );
				$courses[] = $posts[ 'course_id' ];
				learndash_user_set_enrolled_courses( $user_id, $courses );
				$order_details = [
					'step_id'        => $step_id,
					'funnel_id'      => $funnel_id,
					'user_id'        => $user_id,
					'course_details' => Wpfnl_lms_learndash_functions::get_course_details_by_id( $posts[ 'course_id' ] ),
				];
				$this->create_lms_order( $order_details, 'free' );
				$next_step = Wpfnl_lms_learndash_functions::get_next_step_url( $funnel_id, $step_id );
				$response  = [
					'success'   => true,
					'next_step' => $next_step
				];
			}
		}
		wp_send_json( $response );
	}

	/**
	 * Modify registration url
	 * 
	 * @param String
	 * @return String
	 * @since 1.0.3
	 * 
	 */
	public function wpfnl_learndash_login_model_register_url( $url ){

		// $funnel_id = get_post_meta(get_the_ID(), '_funnel_id', true );
		// if( $funnel_id ){
		// 	$url .= '?is_wpfunnel=yes&funnel_id='.$funnel_id.'&step_id='.get_the_ID();
		// }
		return $url;
	}


	/**
	 * Get product info with discounted price
	 * 
	 * @return Array
	 * @since 1.0.3
	 */
	public function wpfnl_get_product_info( $payload ){
		
		$step_id = isset($payload['step_id']) ? $payload['step_id'] : '';
		if( $step_id ){
			$step_type = get_post_meta( $step_id, '_step_type', true );
			if( 'checkout' == $step_type ){
				$discount = get_post_meta( $step_id, '_wpfnl_checkout_discount_main_product', true );
				if( is_array($discount) ){
					if( isset($discount['discountOptions']) && isset($discount['mutedDiscountValue']) && 'original' != $discount['discountOptions'] && $discount['mutedDiscountValue'] ){
						if( isset($discount['discountedPrice']) ){
							$discounted_price = $discount['discountedPrice'];
						}
					}
					
				}
			}
		}
		return [
			'success' => true,
		];
	}


	/**
	 * 
	 * Add discount price to course
	 * 
	 * @param String 
	 * @return String
	 * @since 1.0.4
	 */
    public function wpfnl_learndash_get_price_by_coupon( $price ) {
        if( empty( $_COOKIE[ 'wpfnl_ld_order_bump_accepted' ] ) ) {
            $step_id   = self::get_wpfnl_step_id();
            $step_type = get_post_meta( $step_id, '_step_type', true );

            if( $step_type ) {
                if( 'checkout' == $step_type ) {
                    $discount = get_post_meta( $step_id, '_wpfnl_checkout_discount_main_product', true );
                    if( is_array( $discount ) && isset( $discount[ 'discountOptions' ], $discount[ 'mutedDiscountValue' ], $discount[ 'discountedPrice' ] ) && 'original' != $discount[ 'discountOptions' ] && $discount[ 'mutedDiscountValue' ] ) {
                        $price = $discount[ 'discountedPrice' ];
                    }
                }

                if( 'upsell' === $step_type || 'downsell' === $step_type ) {
                    $discount = get_post_meta( $step_id, '_wpfnl_' . $step_type . '_discount', true );
                    $course   = Wpfnl_lms_learndash_functions::get_course_details( $step_id );
                    if( is_array( $discount ) && isset( $discount[ 'discountType' ], $discount[ 'discountValue' ] ) && 'original' != $discount[ 'discountType' ] && $discount[ 'discountValue' ] ) {
                        $price = Wpfnl_functions::calculate_discount_price( $discount[ 'discountType' ], $discount[ 'discountValue' ], $course[ 'price' ] );
                    }
                }
            }
        }
        else {
            unset( $_COOKIE[ 'wpfnl_ld_order_bump_accepted' ] );
            setcookie( 'wpfnl_ld_order_bump_accepted', '', -1, '/' );
        }
        return (float)str_replace( ',', '', $price );
    }
}
